#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

from keystonemiddleware import auth_token
from oslo_config import cfg
from oslo_log import log as logging
from pecan import hooks

from solum.common import context
from solum.i18n import _


LOG = logging.getLogger(__name__)

OPT_GROUP_NAME = 'keystone_authtoken'

AUTH_OPTS = [
    cfg.BoolOpt('enable_authentication',
                default=True,
                help='This option enables or disables user authentication '
                'via keystone. Default value is True.'),
]


def list_opts():
    yield None, AUTH_OPTS


CONF = cfg.CONF
CONF.register_opts(AUTH_OPTS)

PUBLIC_ENDPOINTS = [
    '^/?$',
    '^/v[0-9]+/?$',
    '^/v[0-9]+/triggers',
    '^/camp/platform_endpoints',
    '^/camp/camp_v1_1_endpoint'
]


def install(app, conf):
    if conf.get('enable_authentication'):
        return AuthProtocolWrapper(app, conf=dict(conf.get(OPT_GROUP_NAME)))
    else:
        LOG.warning(_('Keystone authentication is disabled by Solum '
                      'configuration parameter enable_authentication. '
                      'Solum will not authenticate incoming request. '
                      'In order to enable authentication set '
                      'enable_authentication option to True.'))

    return app


class AuthHelper(object):
    """Helper methods for Auth."""

    def __init__(self):
        endpoints_pattern = '|'.join(pe for pe in PUBLIC_ENDPOINTS)
        self._public_endpoints_regexp = re.compile(endpoints_pattern)

    def is_endpoint_public(self, path):
        return self._public_endpoints_regexp.match(path)


class AuthProtocolWrapper(auth_token.AuthProtocol):
    """A wrapper on Keystone auth_token AuthProtocol.

    Does not perform verification of authentication tokens for pub routes in
    the API. Public routes are those defined by PUBLIC_ENDPOINTS

    """

    def __call__(self, env, start_response):
        path = env.get('PATH_INFO')
        if AUTH.is_endpoint_public(path):
            return self._app(env, start_response)
        return super(AuthProtocolWrapper, self).__call__(env, start_response)


class ContextHook(hooks.PecanHook):

    def before(self, state):
        if not CONF.get('enable_authentication'):
            return
        # Skip authentication for public endpoints
        if AUTH.is_endpoint_public(state.request.path):
            return

        headers = state.request.headers
        user_id = headers.get('X-User-Id')
        roles = self._get_roles(state.request)
        project_id = headers.get('X-Project-Id')
        user_name = headers.get('X-User-Name', '')
        tenant_name = headers.get('X-Project')

        domain = headers.get('X-Domain-Name')
        project_domain_id = headers.get('X-Project-Domain-Id', '')
        user_domain_id = headers.get('X-User-Domain-Id', '')
        password = headers.get('X-Password', '')

        recv_auth_token = headers.get('X-Auth-Token',
                                      headers.get(
                                          'X-Storage-Token'))

        auth_token_info = state.request.environ.get('keystone.token_info')
        ctx = context.RequestContext(auth_token=recv_auth_token,
                                     auth_token_info=auth_token_info,
                                     user=user_id,
                                     project_id=project_id,
                                     domain=domain,
                                     user_domain=user_domain_id,
                                     project_domain=project_domain_id,
                                     user_name=user_name,
                                     roles=roles,
                                     password=password,
                                     project_name=tenant_name)
        state.request.security_context = ctx

    def _get_roles(self, req):
        """Get the list of roles."""

        if 'X-Roles' in req.headers:
            roles = req.headers.get('X-Roles', '')
        else:
            # Fallback to deprecated role header:
            roles = req.headers.get('X-Role', '')
            if roles:
                LOG.warning(_("X-Roles is missing. Using deprecated X-Role "
                              "header"))
        return [r.strip() for r in roles.split(',')]


AUTH = AuthHelper()



# Copyright 2015 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import errno
import os

from http import client as http_client
from oslo_log import log as logging
from swiftclient import exceptions as swiftexp

from solum.common import clients
from solum.common import exception as exc


CHUNKSIZE = 65536
TOTAL_RETRIES = 3
Gi = 1024 * 1024 * 1000
LARGE_OBJECT_SIZE = 5 * Gi

LOG = logging.getLogger(__name__)


class SwiftClient(object):
    """Swift client wrapper so we can encapsulate logic in one place.

       Most of the code in this class are borrowed from
       glance_store._drivers.swift.store.
       """

    def __init__(self, context):
        self.context = context

    def _get_swift_client(self):
        # Create a new non-cached swift client/connection
        return clients.OpenStackClients(self.context).swift()

    def _get_file_size(self, file_obj):
        # Analyze file-like object and attempt to determine its size.

        if (hasattr(file_obj, 'seek') and hasattr(file_obj, 'tell') and
                file_obj.seekable()):
            try:
                curr = file_obj.tell()
                file_obj.seek(0, os.SEEK_END)
                size = file_obj.tell()
                file_obj.seek(curr)
                return size
            except IOError as e:
                if e.errno == errno.ESPIPE:
                    # Illegal seek. This means the file object
                    # is a pipe (e.g. the user is trying
                    # to pipe image data to the client,
                    # echo testdata | bin/glance add blah...), or
                    # that file object is empty, or that a file-like
                    # object which doesn't support 'seek/tell' has
                    # been supplied.
                    return 0
                else:
                    raise
        else:
            return 0

    def _get_object(self, container, name, start_byte=None):
        connection = self._get_swift_client()
        headers = {}
        if start_byte is not None:
            bytes_range = 'bytes=%d-' % start_byte
            headers = {'Range': bytes_range}

        try:
            resp_headers, resp_body = connection.get_object(
                container=container, obj=name, resp_chunk_size=CHUNKSIZE,
                headers=headers)
        except swiftexp.ClientException as e:
            if e.http_status == http_client.NOT_FOUND:
                LOG.debug("Swift could not find object %s." % name)
            raise

        return (resp_headers, resp_body)

    def _retry_iter(self, resp_iter, length, container, name):
        length = length if length else (resp_iter.len
                                        if hasattr(resp_iter, 'len') else 0)
        retries = 0
        bytes_read = 0

        while retries <= TOTAL_RETRIES:
            try:
                for chunk in resp_iter:
                    yield chunk
                    bytes_read += len(chunk)
            except swiftexp.ClientException as e:
                LOG.debug("Swift exception %s" % e.__class__.__name__)
                pass

            if bytes_read == length:
                break
            else:
                if retries == TOTAL_RETRIES:
                    raise exc.MaxRetryReached
                else:
                    retries += 1
                    # NOTE(james_li): Need a new swift connection to do
                    # a range request for the same object
                    (_resp_headers, resp_iter) = self._get_object(
                        container, name, start_byte=bytes_read)

    def upload(self, path, container, name):
        connection = self._get_swift_client()
        with open(path, 'rb') as local_file:
            size = self._get_file_size(local_file)
            if size > 0 and size < LARGE_OBJECT_SIZE:
                connection.put_container(container)
                connection.put_object(container, name, local_file,
                                      content_length=size)
            else:
                raise exc.InvalidObjectSizeError

    def download(self, path, container, name):
        (resp_headers, resp_data) = self._get_object(container, name)
        length = int(resp_headers.get('content-length', 0))
        data_iter = self._retry_iter(resp_data, length, container, name)

        with open(path, 'wb') as local_file:
            for chunk in data_iter:
                local_file.write(chunk)
            local_file.flush()

    def delete_object(self, container, filename):
        swift = self._get_swift_client()
        try:
            swift.delete_object(container, filename)
        except swiftexp.ClientException as e:
            if e.http_status == http_client.NOT_FOUND:
                LOG.debug("Swift could not find object %s." % filename)
                pass
            else:
                raise

    def delete_container(self, container):
        swift = self._get_swift_client()
        swift.delete_container(container)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import datetime as dt
import json
import logging
import time

from oslo_config import cfg
from oslo_log import log as os_logging

import solum.uploaders.local as local_uploader
import solum.uploaders.swift as swift_uploader

cfg.CONF.import_opt('log_upload_strategy', 'solum.worker.config',
                    group='worker')
LOG = os_logging.getLogger(__name__)


class TenantLogger(object):

    def __init__(self, ctxt, assem, workflow_id, deployer_log_dir, stage):
        strategy = cfg.CONF.worker.log_upload_strategy
        LOG.debug("User log upload strategy: %s" % strategy)

        self.ctxt = ctxt
        self.assem = assem
        self.stage = stage

        # Note: assembly type is used by uploader
        self.assem.type = 'app'

        tenant_log_file = "%s-%s" % (stage, workflow_id)
        self.path = "%s/%s.log" % (deployer_log_dir, tenant_log_file)
        LOG.debug("Deployer logs stored at %s" % self.path)

        uploadr = {
            'local': local_uploader.LocalStorage,
            'swift': swift_uploader.SwiftUpload,
        }.get(strategy, local_uploader.LocalStorage)

        self.uploader = uploadr(ctxt, self.path, assem, workflow_id, stage)

        self.assem_logger = logging.getLogger(workflow_id)
        self.assem_logger.setLevel(logging.DEBUG)

        self.handler = logging.FileHandler(self.path, "a", encoding=None,
                                           delay="false")
        self.handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        self.handler.setFormatter(formatter)
        self.assem_logger.addHandler(self.handler)

    def upload(self):
        self.handler.close()
        self.uploader.upload_log()

    def log(self, level, message):
        logline = self._logline(message)
        try:
            self.assem_logger.log(level, logline)
        except IOError as e:
            LOG.error(e)

    def _logline(self, message):
        log_line = {}
        t = time.time()
        print_tm = dt.datetime.fromtimestamp(t).strftime('%Y-%m-%d %H:%M:%S')
        log_line['@timestamp'] = print_tm
        log_line['project_id'] = self.ctxt.project_id
        log_line['stage_id'] = self.assem.uuid
        log_line['task'] = self.stage
        log_line['message'] = message
        return json.dumps(log_line)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import solum
from solum.common import context
from solum.common import trace_data
from solum.tests import base

solum.TLS.trace = trace_data.TraceData()

# Just putting highly recognizable values in context
CONTEXT = context.RequestContext(auth_token='_auth_token_',
                                 user='_user_',
                                 project_id='_project_id_',
                                 domain='_domain_',
                                 user_domain='_user_domain_',
                                 project_domain='_project_domain_',
                                 is_admin=False,
                                 read_only=False,
                                 request_id='_request_id_',
                                 user_name='_user_name_',
                                 roles='_roles_',
                                 auth_url='_auth_url_',
                                 auth_token_info='_auth_token_info_')


class TestTraceData(base.BaseTestCase):
    """Tests the TraceData class."""
    def test_auto_clear(self):
        """auto_clear success and then a failure case."""
        solum.TLS.trace.auto_clear = True
        solum.TLS.trace.auto_clear = False
        try:
            solum.TLS.trace.auto_clear = 'fail'
        except AssertionError:
            pass
        else:
            self.assertTrue(False)

    def test_import_context(self):
        """Test importing Oslo RequestContext."""
        solum.TLS.trace.clear()
        solum.TLS.trace.import_context(CONTEXT)
        self.assertEqual(
            solum.TLS.trace._user_data,
            {'user': '_user_', 'project_id': '_project_id_'})
        self.assertEqual(({
            'auth_token_info': '_auth_token_info_',
            'auth_url': '_auth_url_',
            'domain': '_domain_',
            'global_request_id': None,
            'is_admin': False,
            'is_admin_project': True,
            'project': '_project_id_',
            'project_domain': '_project_domain_',
            'read_only': False,
            'resource_uuid': None,
            'roles': '_roles_',
            'show_deleted': False,
            'system_scope': None,
            'user_domain': '_user_domain_',
            'user_identity': u'_user_ _project_id_ _domain_ '
            '_user_domain_ _project_domain_',
            'user_name': '_user_name_'}), solum.TLS.trace._support_data)

    def test_info_commands(self):
        """Test trace setting functions."""
        solum.TLS.trace.clear()
        solum.TLS.trace.request_id = '98765'
        solum.TLS.trace.user_info(ip_addr="1.2.3.4", user_id=12345)
        solum.TLS.trace.support_info(confidential_data={"a": "b", "c": "d"})
        self.assertEqual(
            solum.TLS.trace._user_data,
            {'ip_addr': '1.2.3.4', 'user_id': 12345})
        self.assertEqual(
            solum.TLS.trace._support_data,
            {'confidential_data': {'a': 'b', 'c': 'd'}})
        self.assertEqual(
            solum.TLS.trace.request_id,
            '98765')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import core
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import type_definitions as model
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.handlers.camp import type_definition_handler
from solum.common import exception


DESCRIPTION_STRING = "Solum CAMP API type_definitions collection resource."


class TypeDefinitionsController(rest.RestController):
    """CAMP v1.1 type_definitions controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.TypeDefinition, wtypes.text)
    def get_one(self, type_def_name):
        handler = (type_definition_handler.
                   TypeDefinitionHandler(pecan.request.security_context))
        raw_def = handler.get(type_def_name)
        if not raw_def:
            core.abort(404,
                       '%s is not a type_definition' %
                       type_def_name)
        host_url = pecan.request.application_url.rstrip('/')
        return raw_def.fix_uris(host_url)

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.TypeDefinitions)
    def get(self):
        links = []
        handler = (type_definition_handler.
                   TypeDefinitionHandler(pecan.request.security_context))
        def_list = handler.get_all()

        host_url = pecan.request.application_url.rstrip('/')
        for tdef in def_list:
            links.append(common_types.Link(href=uris.TYPE_DEF_URI_STR %
                                           (host_url, tdef.uri),
                                           target_name=tdef.name))

        return model.TypeDefinitions(uri=uris.TYPE_DEFS_URI_STR % host_url,
                                     name='Solum_CAMP_type_definitions',
                                     type='type_definitions',
                                     description=DESCRIPTION_STRING,
                                     type_definition_links=links)



# Copyright 2015 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.common import exception as exc
from solum.common import solum_swiftclient as swiftclient
from solum.tests import base
from solum.tests import utils


class SwiftClientTest(base.BaseTestCase):
    """Test cases for solum.common.solum_swiftclient."""

    @mock.patch('builtins.open')
    @mock.patch('solum.common.solum_swiftclient.SwiftClient._get_swift_client')
    @mock.patch('solum.common.solum_swiftclient.SwiftClient._get_file_size')
    def test_swift_client_upload(self, mock_file_size, mock_swift_client,
                                 mock_open):

        ctxt = utils.dummy_context()
        container = 'fake-container'
        filename = 'fake-file'
        mock_client = mock_swift_client.return_value
        fsize = 5
        mock_file_size.return_value = fsize

        swift = swiftclient.SwiftClient(ctxt)
        swift.upload('filepath', container, filename)
        mock_client.put_container.assert_called_once_with(container)
        mock_client.put_object.assert_called_once_with(container,
                                                       filename,
                                                       mock.ANY,
                                                       content_length=fsize)

    @mock.patch('builtins.open')
    @mock.patch('solum.common.solum_swiftclient.SwiftClient._get_swift_client')
    @mock.patch('solum.common.solum_swiftclient.SwiftClient._get_file_size')
    def test_swift_client_upload_exception(self, mock_file_size,
                                           mock_swift_client, mock_open):

        ctxt = utils.dummy_context()
        mock_file_size.return_value = 0

        swift = swiftclient.SwiftClient(ctxt)
        self.assertRaises(exc.InvalidObjectSizeError,
                          swift.upload, 'filepath', 'fake-container', 'fname')



# Copyright 2015 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""LP handler for building apps running on solum language packs"""

import io
import logging
import os
import random
import string

from docker import errors
from oslo_config import cfg
from oslo_log import log as os_logging

from solum.common import clients
from solum.common import solum_swiftclient
from solum.worker.app_handlers import base
from solum.worker.app_handlers import utils

from swiftclient import exceptions as swiftexp


LOG = os_logging.getLogger(__name__)

cfg.CONF.import_opt('container_mem_limit', 'solum.worker.config',
                    group='worker')
mem_limit = cfg.CONF.worker.container_mem_limit
build_timeout = cfg.CONF.worker.docker_build_timeout
UNITTEST_TIMEOUT = 1800  # 30 minutes


class DockerHandler(base.BaseHandler):

    def __init__(self, context, assembly, lp_type, image_storage):
        super(DockerHandler, self).__init__(context, assembly, image_storage)

        self.lp_type = lp_type
        self.lp = None
        self.source_sha = None
        if self.image_storage == 'glance':
            self.glance = clients.OpenStackClients(context).glance()

    def _download_lp(self, lp_obj_name, lp_img_tag, logger):
        # TODO(james_li): try cache before downloading from origin
        logger.log(logging.INFO, 'Downloading LP...')

        if self.image_storage == 'glance':
            return
        elif self.image_storage == 'docker_registry':
            return
        elif self.image_storage == 'swift':
            if self.work_dir is None:
                return
            path = '{}/lp/{}'.format(self.work_dir, lp_obj_name)
            swift = solum_swiftclient.SwiftClient(self.context)
            try:
                swift.download(path, 'solum_lp', lp_obj_name)
            except swiftexp.ClientException as e:
                LOG.error('Error in downloading LP %s, %s' %
                          (lp_obj_name, str(e)))
                logger.log(logging.ERROR, 'Downloading LP failed.')
                return

            if self._docker_load(path) != 0:
                logger.log(logging.ERROR, 'Loading docker image failed.')
                return

            logger.log(logging.INFO, 'LP downloaded and loaded successfully.')
            return lp_img_tag

    def _prepare(self, git_info, lp_obj_name, lp_img_tag, logger):
        """Create working dir and download LP only once for an app workflow."""
        if self.work_dir is None or self.source_sha is None:
            tenant = self.context.project_id
            self.work_dir = '/tmp/apps/{tenant}/{id}'.format(
                tenant=tenant, id=self.assembly.uuid)
            try:
                os.makedirs(self.work_dir)
                os.chmod(self.work_dir, 0o774)
                os.mkdir('{}/lp'.format(self.work_dir))
            except OSError as e:
                LOG.error('Error creating app dir %s, %s' %
                          (self.work_dir, str(e)))
                logger.log(logging.ERROR, 'Building app preparation failed.')
                return False

            revision = git_info.get('revision', 'master')

            head_sha = self._clone_repo(git_info['source_url'], self.work_dir,
                                        logger, revision=revision)
            if not head_sha:
                logger.log(logging.ERROR, 'Failed cloning app repo %s.' %
                           git_info['source_url'])
                return False

            self.source_sha = head_sha
            self._gen_docker_ignore(self.work_dir, 'code')

        if self.lp is None:
            self.lp = self._download_lp(lp_obj_name, lp_img_tag, logger)
            if self.lp is None:
                return False

        return True

    def build_lp(self, lp_name, git_info):
        logger = self._get_tenant_logger('language_pack')
        tenant = self.context.project_id
        ts = utils.timestamp()
        ranid = (''.join(random.choice(string.ascii_uppercase)
                 for _ in range(20)))
        self.work_dir = '/tmp/lps/{tenant}/{id}'.format(tenant=tenant,
                                                        id=ranid)
        try:
            os.makedirs(self.work_dir)
            os.chmod(self.work_dir, 0o774)
        except OSError as e:
            LOG.error('Error creating working dir %s, %s' %
                      (self.work_dir, str(e)))
            logger.log(logging.ERROR, 'Building LP preparation failed.')
            logger.upload()
            return

        revision = git_info.get('revision', 'master')
        head_sha = self._clone_repo(git_info['source_url'], self.work_dir,
                                    logger, revision=revision)
        if not head_sha:
            logger.log(logging.ERROR, 'Failed cloning LP repo %s.' %
                       git_info['source_url'])
            logger.upload()
            return

        storage_obj_name = '{name}-{ts}-{sha}'.format(name=lp_name, ts=ts,
                                                      sha=head_sha)
        lp_image_tag = '{tenant}-{obj}'.format(tenant=tenant,
                                               obj=storage_obj_name)
        dockerfile = '{}/code'.format(self.work_dir)

        logger.log(logging.INFO, 'Start building LP...')
        result = self._docker_build_with_retry(lp_image_tag, logger,
                                               path=dockerfile)
        if result != 0:
            logger.log(logging.ERROR, 'Failed building LP image.')
            logger.upload()
            return

        lp_file = '{}/{}'.format(self.work_dir, storage_obj_name)
        result = self._docker_save(lp_image_tag, lp_file)
        if result != 0:
            logger.log(logging.ERROR, 'Failed saving LP image.')
            logger.upload()
            return

        image_loc = self._persist_to_backend(lp_file, 'solum_lp',
                                             storage_obj_name, logger)
        if image_loc is None:
            logger.log(logging.ERROR, 'Failed persisting LP to backend.')
            logger.upload()
            return
        else:
            logger.log(logging.INFO, 'Successfully created LP image.')
            logger.upload()
            return (image_loc, lp_image_tag)

    def unittest_app(self, git_info, lp_obj_name, lp_img_tag, test_cmd):
        logger = self._get_tenant_logger('unittest')
        if not self._prepare(git_info, lp_obj_name, lp_img_tag, logger):
            logger.upload()
            return -1

        timeout_cmd = ('timeout --signal=SIGKILL {t}'
                       ' /bin/sh -c \"{test}\"').format(t=UNITTEST_TIMEOUT,
                                                        test=test_cmd)
        # username = (''.join(random.choice(string.ascii_lowercase)
        #                    for _ in range(8)))
        # useradd_cmd = ('useradd -s /bin/bash -u {uid} -m {uname} ||'
        #               ' usermod -d /app $(getent passwd {uid}'
        #               ' | cut -d: -f1)').format(uid=self.docker_cmd_uid,
        #                                         uname=username)

        # Will run user's arbitrary test_cmd as root in container,
        # waiting for the following docker patch to remap the root in
        # a container to an unprivileged user on host:
        # https://github.com/docker/docker/pull/12648
        # If the docker patch is finally abandoned, we should run test_cmd as
        # unprivileged by using the commented code above, in which case
        # we may want to leverage the following docker feature:
        # https://github.com/docker/docker/pull/10775/commits
        content = ('FROM {lp}\n'
                   'COPY code /app\n'
                   'WORKDIR /app\n'
                   'CMD {cmd}').format(lp=self.lp, cmd=timeout_cmd)
        df = 'Dockerfile.ut'
        fname = '{}/{}'.format(self.work_dir, df)
        try:
            with open(fname, 'w') as f:
                f.write(content)
        except OSError as e:
            LOG.error('Error in creating Dockerfile %s, %s' % (fname, str(e)))
            logger.log(logging.ERROR, 'Preparing running unittest failed')
            logger.upload()
            return -1

        logger.log(logging.INFO, 'Building image for running unittests...')
        tag = 'unittest-{}'.format(self.assembly.uuid)
        build_result = self._docker_build_with_retry(tag, logger,
                                                     path=self.work_dir,
                                                     dockerfile=df,
                                                     pull=False)
        self.images.append(tag)
        if build_result != 0:
            logger.log(logging.ERROR, 'Failed building image to run unittest.')
            logger.upload()
            return -1

        ct = None
        logger.log(logging.INFO, 'Running unittests...')
        try:
            ct = self.docker.create_container(image=tag, mem_limit=mem_limit,
                                              memswap_limit=-1)
            self.containers.append(ct)
            self.docker.start(container=ct.get('Id'))
            result = self.docker.wait(container=ct.get('Id'))
        except (errors.DockerException, errors.APIError) as e:
            LOG.error('Error running unittest, assembly: %s, %s' %
                      (self.assembly.uuid, str(e)))
            logger.log(logging.ERROR, 'Running unittest failed')
            logger.upload()
            return -1

        logger.log(logging.INFO, 'Finished unit testing with return code %s.' %
                   result)
        logger.upload()
        return result

    def build_app(self, app_name, git_info, lp_obj_name, lp_img_tag,
                  run_cmd):
        logger = self._get_tenant_logger('build')
        if not self._prepare(git_info, lp_obj_name, lp_img_tag, logger):
            logger.upload()
            return

        timeout_cmd = ('timeout --signal=SIGKILL {t} {cmd}').format(
            t=build_timeout, cmd='./build.sh')

        # username = (''.join(random.choice(string.ascii_lowercase)
        #                    for _ in range(8)))
        # useradd_cmd = ('useradd -s /bin/bash -u {uid} -m {uname} ||'
        #               ' usermod -d /app $(getent passwd {uid}'
        #               ' | cut -d: -f1)').format(uid=self.docker_cmd_uid,
        #                                         uname=username)

        # Will run user's arbitrary build.sh as root in container,
        # waiting for the following docker patch to remap the root in
        # a container to an unprivileged user on host:
        # https://github.com/docker/docker/pull/12648
        # If the docker patch is finally abandoned, we should run build.sh as
        # unprivileged by using the commented code above, in which case
        # we may want to leverage the following docker feature:
        # https://github.com/docker/docker/pull/10775/commits
        content_build = ('FROM {lp}\n'
                         'COPY code /app\n'
                         'WORKDIR /solum/bin\n'
                         'RUN chmod +x build.sh\n'
                         'CMD {cmd}').format(lp=self.lp, cmd=timeout_cmd)
        df = 'Dockerfile.build'
        fname = '{}/{}'.format(self.work_dir, df)
        try:
            with open(fname, 'w') as f:
                f.write(content_build)
        except OSError as e:
            LOG.error('Error in creating Dockerfile %s, %s' % (fname, str(e)))
            logger.log(logging.ERROR, 'Preparing building app DU image failed')
            logger.upload()
            return

        tenant = self.context.project_id
        ts = utils.timestamp()
        storage_obj_name = '{name}-{ts}-{sha}'.format(name=app_name, ts=ts,
                                                      sha=self.source_sha)
        du_image = '{tenant}-{obj}'.format(tenant=tenant,
                                           obj=storage_obj_name)
        du_image_in_build = '{img}:{tag}'.format(img=du_image, tag='build')

        logger.log(logging.INFO, 'Building DU image, preparing to run'
                                 ' build script')
        build_result = self._docker_build_with_retry(du_image_in_build, logger,
                                                     path=self.work_dir,
                                                     dockerfile=df,
                                                     pull=False)
        self.images.append(du_image_in_build)
        if build_result != 0:
            logger.log(logging.ERROR, 'Failed building DU image.')
            logger.upload()
            return

        logger.log(logging.INFO, 'Building DU image, running build script')
        ct = None
        result = -1
        try:
            ct = self.docker.create_container(image=du_image_in_build,
                                              mem_limit=mem_limit,
                                              memswap_limit=-1)
            self.containers.append(ct)
            self.docker.start(container=ct.get('Id'))
            result = self.docker.wait(container=ct.get('Id'))
        except (errors.DockerException, errors.APIError) as e:
            LOG.error('Error running build script, assembly: %s, %s' %
                      (self.assembly.uuid, str(e)))
            logger.log(logging.ERROR, 'Running build script failed')
            logger.upload()
            return

        if result != 0:
            logger.log(logging.ERROR, 'Build script returns with %s' % result)
            logger.upload()
            return

        try:
            self.docker.commit(container=ct.get('Id'), repository=du_image,
                               tag='build')
        except (errors.DockerException, errors.APIError) as e:
            LOG.error('Error committing the built image layer from build'
                      ' script, assembly: %s, %s' %
                      (self.assembly.uuid, str(e)))
            logger.log(logging.ERROR, 'Error building DU with the output'
                                      ' of build script')
            logger.upload()
            return

        content_run = ('FROM {img}\n'
                       'WORKDIR /app\n'
                       'CMD {cmd}').format(img=du_image_in_build, cmd=run_cmd)
        f = io.BytesIO(content_run.encode('utf-8'))
        build_result = self._docker_build_with_retry(du_image, logger,
                                                     fileobj=f, pull=False)
        self.images.append(du_image)
        if build_result != 0:
            logger.log(logging.ERROR, 'Failed building DU image.')
            logger.upload()
            return

        du_file = '{}/{}'.format(self.work_dir, storage_obj_name)
        result = self._docker_save(du_image, du_file)
        if result != 0:
            logger.log(logging.ERROR, 'Failed saving DU image.')
            logger.upload()
            return

        logger.log(logging.INFO, 'Persisting DU image to backend')
        image_loc = self._persist_to_backend(du_file, 'solum_du',
                                             storage_obj_name, logger)
        if image_loc is None:
            logger.log(logging.ERROR, 'Failed persist DU to backend.')
            logger.upload()
            return
        else:
            logger.log(logging.INFO, 'Successfully created DU image.')
            logger.upload()
            return (image_loc, du_image)



#    Copyright 2013 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class Registry(object):
    impls = dict()

    """Allow domain objects to be loaded by name."""
    def __getattr__(self, name):
        if name in self.impls:
            return self.impls[name]

    def add(self, interface, cls):
        """Register an implementation for a class."""
        self.impls[interface.__name__] = cls

    def clear(self):
        """Deregister all implementations."""
        self.impls.clear()



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import operation as operationmodel
from solum.api.controllers.v1 import operation
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.operation_handler.OperationHandler')
class TestOperationController(base.BaseTestCase):

    def setUp(self):
        super(TestOperationController, self).setUp()
        objects.load()

    def test_operation_get(self, OperationHandler, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = OperationHandler.return_value.get
        fake_operation = fakes.FakeOperation()
        hand_get.return_value = fake_operation
        cont = operation.OperationController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_operation.name, resp['result'].name)
        self.assertEqual(fake_operation.documentation,
                         resp['result'].documentation)
        self.assertEqual(fake_operation.description,
                         resp['result'].description)
        self.assertEqual(fake_operation.project_id, resp['result'].project_id)
        self.assertEqual(fake_operation.uuid, resp['result'].uuid)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_operation_get_not_found(self, OperationHandler, resp_mock,
                                     request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = OperationHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(
            id='test_id', name='operation')
        cont = operation.OperationController('test_id')
        cont.get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_operation_put_none(self, OperationHandler, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        hand_put = OperationHandler.return_value.put
        hand_put.return_value = fakes.FakeOperation()
        operation.OperationController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_operation_put_not_found(self, OperationHandler, resp_mock,
                                     request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = OperationHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            id='test_id', name='operation')
        operation.OperationController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_operation_put_ok(self, OperationHandler, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = OperationHandler.return_value.update
        hand_update.return_value = fakes.FakeOperation()
        operation.OperationController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(200, resp_mock.status)

    def test_operation_delete_not_found(self, OperationHandler,
                                        resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = OperationHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            id='test_id', name='operation')
        obj = operation.OperationController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_operation_delete_ok(self, OperationHandler, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = OperationHandler.return_value.delete
        hand_delete.return_value = None
        obj = operation.OperationController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.operation_handler.OperationHandler')
class TestOperationsController(base.BaseTestCase):
    def setUp(self):
        super(TestOperationsController, self).setUp()
        objects.load()

    def test_operations_get_all(self, handler_mock, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get_all = handler_mock.return_value.get_all
        fake_operation = fakes.FakeOperation()
        hand_get_all.return_value = [fake_operation]
        obj = operation.OperationsController()
        resp = obj.get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_operation.name, resp['result'][0].name)
        self.assertEqual(fake_operation.documentation,
                         resp['result'][0].documentation)
        self.assertEqual(fake_operation.description,
                         resp['result'][0].description)
        self.assertEqual(fake_operation.project_id,
                         resp['result'][0].project_id)
        self.assertEqual(fake_operation.uuid, resp['result'][0].uuid)
        self.assertEqual(200, resp_mock.status)

    def test_operations_post(self, handler_mock, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'description': 'test_desc_operation',
                       'user_id': 'user_id_test',
                       'project_id': 'project_id_test'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeOperation()
        operation.OperationsController().post()
        handler_create.assert_called_with(json_create)
        self.assertEqual(201, resp_mock.status)
        handler_create.assert_called_once_with(json_create)

    def test_operations_post_nodata(self, handler_mock,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeComponent()
        ret_val = operation.OperationsController().post()
        self.assertEqual("Missing argument: \"data\"",
                         str(ret_val['faultstring']))
        self.assertEqual(400, resp_mock.status)


class TestOperationAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/operations/x1',
                            'name': 'Example-operation',
                            'type': 'operation',
                            'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                            'user_id': '55f41cf46df74320b9486a35f5d28a11',
                            'documentation':
                            'http://example.com/docs/resume_op',
                            'target_resource':
                            'http://example.com/instances/uuid'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = operationmodel.Operation()
            self.data = {}
        else:
            s = operationmodel.Operation(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']
        self.assertEqual(self.data, s.as_dict(objects.registry.Operation))



# Copyright 2013 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import component
from solum.api.controllers.v1.datamodel import operation
from solum.api.controllers.v1.datamodel import sensor
from solum.api.controllers.v1.datamodel import types as api_types


class Assembly(api_types.Base):
    """Representation of an Assembly.

    The Assembly resource represents a group of components that make
    up a running instance of an application. You may casually refer
    to this as "the application" but we refer to it as an Assembly because
    most cloud applications are actually a system of multiple service
    instances that make up a system. For example, a three-tier web
    application may have a load balancer component, a group of application
    servers, and a database server all represented as Component resources
    that make up an Assembly resource. An Assembly resource has at least
    one Component resource associated with it.
    """

    plan_uri = common_types.Uri
    """The URI to the plan to be used to create this assembly."""

    components = [component.Component]
    """Components that belong to the assembly."""

    operations = [operation.Operation]
    """Operations that belong to the assembly."""

    sensors = [sensor.Sensor]
    """Sensors that belong to the assembly."""

    status = wtypes.text
    """The status of the assembly."""

    application_uri = common_types.Uri
    """The uri of the deployed application."""

    workflow = [wtypes.Enum(wtypes.text, 'unittest', 'build', 'deploy')]
    """Defines the workflow that an assembly will go through."""

    updated_at = datetime.datetime
    """The last time a change was made to the assembly's status."""

    created_at = datetime.datetime
    """The time the assembly initially created."""

    @classmethod
    def from_db_model(cls, m, host_url):
        obj = super(Assembly, cls).from_db_model(m, host_url)
        obj.plan_uri = '%s/v1/plans/%s' % (host_url, m.plan_uuid)
        return obj

    @classmethod
    def sample(cls):
        now = datetime.datetime.utcnow()
        return cls(uri='http://example.com/v1/assemblies/x4',
                   name='database',
                   type='assembly',
                   plan_uri='http://example.com/v1/plans/45-09',
                   tags=['small'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='A mysql database',
                   components=[],
                   operations=[],
                   created_at=now,
                   updated_at=now,
                   sensors=[])



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sa
from sqlalchemy.orm import exc
from sqlalchemy.sql.expression import desc

from solum import objects
from solum.objects import assembly as abstract
from solum.objects.sqlalchemy import component
from solum.objects.sqlalchemy import models as sql
from solum.objects.sqlalchemy import plan as plan_obj

ASSEMBLY_STATES = abstract.States
retry = sql.retry


class Assembly(sql.Base, abstract.Assembly):
    """Represent an assembly in sqlalchemy."""

    __tablename__ = 'assembly'
    __resource__ = 'assemblies'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    uuid = sa.Column(sa.String(36), nullable=False)
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))
    name = sa.Column(sa.String(100))
    description = sa.Column(sa.String(255))
    tags = sa.Column(sa.Text)
    plan_id = sa.Column(sa.Integer, sa.ForeignKey('plan.id'), nullable=False)
    status = sa.Column(sa.String(36))
    application_uri = sa.Column(sa.String(1024))
    username = sa.Column(sa.String(256))
    workflow = sa.Column(sql.YAMLEncodedDict(1024))
    image_id = sa.Column(sa.Integer)

    def _non_updatable_fields(self):
        return set(('uuid', 'id', 'project_id'))

    def _is_updatable(self):
        if self.status == ASSEMBLY_STATES.DELETING:
            return False
        else:
            return True

    @property
    def plan_uuid(self):
        return objects.registry.Plan.get_by_id(None, self.plan_id).uuid

    @plan_uuid.setter
    def plan_uuid(self, value):
        plan = objects.registry.Plan.get_by_uuid(None, value)
        self.plan_id = plan.id

    @property
    def _extra_keys(self):
        return ['plan_uuid', 'created_at', 'updated_at']

    @property
    def components(self):
        session = sql.Base.get_session()
        return session.query(component.Component).filter_by(
            assembly_id=self.id).all()

    @retry
    def destroy(self, context):
        session = sql.Base.get_session()
        with session.begin():
            session.query(component.Component).filter_by(
                assembly_id=self.id).delete()
            session.query(self.__class__).filter_by(
                id=self.id).delete()

    @property
    def heat_stack_component(self):
        session = sql.Base.get_session()
        return session.query(component.Component).filter_by(
            assembly_id=self.id, component_type='heat_stack').first()


class AssemblyList(abstract.AssemblyList):
    """Represent a list of assemblies in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        mq = sql.model_query(context, Assembly).order_by(
            desc("updated_at"), desc("created_at"))
        return AssemblyList(mq)

    @classmethod
    def get_earlier(cls, assem_id, app_id, status, created_at):
        try:
            session = sql.Base.get_session()
            result = session.query(plan_obj.Plan)
            result = result.filter(plan_obj.Plan.id == app_id)
            app_uuid = result.one().uuid
            result = session.query(plan_obj.Plan)
            result = result.filter(plan_obj.Plan.uuid == app_uuid).all()
            plan_id_list = [plan.id for plan in result]

            result = session.query(Assembly)
            result = result.filter(Assembly.plan_id.in_(plan_id_list))
            result = result.filter(Assembly.created_at < created_at)
            return result.all()
        except exc.NoResultFound:
            cls._raise_not_found(assem_id)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.api import app as api_app
from solum.api import auth
from solum.api import config as api_config
from solum.tests import base


class TestAppConfig(base.BaseTestCase):
    def setUp(self):
        super(TestAppConfig, self).setUp()

    def test_get_pecan_config(self):
        config = api_app.get_pecan_config()

        config_d = dict(config.app)

        self.assertEqual(config_d['modules'], api_config.app['modules'])
        self.assertEqual(config_d['root'], api_config.app['root'])
        self.assertIsInstance(config_d['hooks'][0], auth.ContextHook)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# The list below holds the keys corresponding to Oslo RequestContext keys
# which are acceptable for an authenticated user to view.  Everything else
# will be considered support-only data.  This list may be too restrictive and
# can be relaxed more as needed.  This is only used if import_context() is
# called.
_TRACE_USER_KEYS = ["user", "project_id"]
_TRACE_CONTEXT_IGNORE = ["auth_token", "trust_id"]


class TraceData(object):
    """This class holds trace data needed for logging.

    This class also provides a way to indicate the confidentiality of stored
    data.  This is useful for properly logging support or operator-only data
    in such a way that the backend log/notification filters have a single key
    to filter on.  It also provides a centralized location to add secure
    storage/cleanup algorithms.
    """
    def __init__(self):
        """Initialize class storage."""
        self.request_id = "<not set>"
        self._auto_clear = False
        self._user_data = {}
        self._support_data = {}

    def import_context(self, context):
        """Accept an Oslo RequestContext and fill in data automatically."""
        context_dict = context.to_dict()
        for key, val in context_dict.items():
            if key == "request_id":
                self.request_id = val
            elif key in _TRACE_USER_KEYS:
                self._user_data[key] = val
            elif key in _TRACE_CONTEXT_IGNORE:
                # these are just too big.
                continue
            else:
                self._support_data[key] = val

    def user_info(self, **kwargs):
        """Add data to user-visible storage."""
        self._user_data.update(kwargs)

    def support_info(self, **kwargs):
        """Add confidential data to support/operator-visible only storage."""
        self._support_data.update(kwargs)

    def clear(self):
        """Clear data regardless of confidential status.

        Note: The lightweight memory clearing below is certainly not what we
        want in the end.  It is just an initial/simple/portable way to do this
        for now.  It is only somewhat useful before garbage collection.
        """
        self.request_id = "<not set>"
        self._user_data = {}
        self._support_data = {}

    def to_dict(self):
        """Generate a dictionary for Oslo Log (requires 'request_id').

        Because this class initializes request_id to <not set> the log
        operation will succeed even if a UUID was not stored.
        """
        if self._auto_clear is not True:
            return ({"request_id": self.request_id,
                     "user_trace": self._user_data,
                     "support_trace": self._support_data})

        user_data = self._user_data.copy()
        support_data = self._support_data.copy()
        self.clear()
        return ({"request_id": self.request_id,
                 "user_trace": user_data,
                 "support_trace": support_data})

    @property
    def auto_clear(self):
        """_auto_clear getter."""
        return self._auto_clear

    @auto_clear.setter
    def auto_clear(self, value):
        """_auto_clear setter to enable assertion checks.

        When set to True, this class will automatically delete all trace data
        after the next Oslo log call (or a to_dict() call).  If the Oslo log
        call does not log any data (log levels preventing it for example) then
        the data is still deleted so care must be taken when using this.
        """
        assert isinstance(value, bool)
        self._auto_clear = value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp import camp_v1_1_endpoint
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
class TestCAMPEndpoint(base.BaseTestCase):
    def setUp(self):
        super(TestCAMPEndpoint, self).setUp()
        objects.load()

    def test_camp_endpoint_get(self, resp_mock, request_mock):
        cont = camp_v1_1_endpoint.CAMPv11EndpointController()
        resp = cont.get()
        self.assertEqual(200, resp_mock.status)
        self.assertEqual('platform_endpoint', resp['result'].type)
        self.assertEqual(camp_v1_1_endpoint.NAME_STRING, resp['result'].name)



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import datetime

import wsme
from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import types as api_types


class App(api_types.Base):
    """Representation of an App.

    The App is the primary resource managed by Solum.
    """
    # Inherited fields:
    # uri
    # uuid
    # name is overridden.
    # type
    # description
    # tags
    # project_id
    # user_id

    version = wtypes.wsattr(int)
    id = wtypes.text
    name = wtypes.text
    deleted = bool
    languagepack = wtypes.text
    stack_id = wtypes.text
    ports = api_types.MultiType(api_types.PortType,
                                [api_types.PortType],
                                {wtypes.text: api_types.PortType})
    source = wtypes.DictType(wtypes.text, api_types.MultiType(
        wtypes.text,
        int,
        bool,
        float))
    workflow_config = {wtypes.text: wtypes.text}
    trigger_uuid = wtypes.text
    trigger_actions = [wtypes.text]
    trigger_uri = wtypes.text
    trust_id = wtypes.text
    trust_user = wtypes.text
    app_url = wtypes.text
    status = wtypes.text
    repo_token = wtypes.text
    created_at = datetime.datetime
    updated_at = datetime.datetime
    raw_content = wtypes.text
    repo_token = wtypes.text
    scale_config = wtypes.DictType(
        wtypes.text,
        wtypes.DictType(wtypes.text, wtypes.text))

    parameters = wtypes.DictType(
        wtypes.text,
        wtypes.DictType(wtypes.text,
                        api_types.MultiType(
                            wtypes.text,
                            int,
                            bool,
                            float)))

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)

    @classmethod
    def sample(cls):
        return cls(
            name="sampleapp",
            description="sample app",
            )

    @classmethod
    def from_db_model(cls, m, host_url):

        json = m.as_dict()
        json['type'] = m.__tablename__
        json['uri'] = '%s/v1/apps/%s' % (host_url, m.id)
        json['trigger_uri'] = ('%s/v1/triggers/%s' %
                               (host_url, m.trigger_uuid))
        return cls(**(json))

    def as_dict(self, db_model):
        attrs = [
            'name',
            'id',
            'description',
            'languagepack',
            'project_id',
            'user_id',
            'deleted',
            'source',
            'ports',
            'trigger_actions',
            'workflow_config',
            'stack_id',
            'raw_content',
            'repo_token'
            ]
        base = super(App, self).as_dict(db_model)

        if self.parameters is not wsme.Unset:
            base.update({'parameters': self.parameters})

        for a in attrs:
            if getattr(self, a) is wsme.Unset:
                continue
            if getattr(self, a) is None:
                continue
            base[a] = getattr(self, a)
        return base



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from wsme import types as wtypes

from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Assembly(api_types.Base):
    """CAMP v1.1 assembly resource model."""

    components = [common_types.Link]
    """CAMP-defined, not currently used."""

    plan_uri = common_types.Uri
    """CAMP-defined, also used by Solum."""

    operations_uri = common_types.Uri
    """CAMP-defined, not currently used."""

    sensors_uri = common_types.Uri
    """CAMP-defined, not currently used."""

    status = wtypes.text
    """Solum extension."""

    updated_at = datetime.datetime
    """Solum extension."""

    created_at = datetime.datetime
    """Solum extension."""

    @classmethod
    def from_db_model(cls, m, host_url):
        obj = super(Assembly, cls).from_db_model(m, host_url)
        obj.plan_uri = uris.PLAN_URI_STR % (host_url, m.plan_uuid)
        obj.uri = uris.ASSEM_URI_STR % (host_url, m.uuid)
        return obj


class Assemblies(api_types.Base):
    """CAMP v1.1 assemblies resource model."""

    assembly_links = [common_types.Link]
    """This attribute contains Links to the assembly resources that represent
     the applications deployed on this platform."""

    parameter_definitions_uri = common_types.Uri
    """The value of the parameter_definitions_uri attribute references a
     resource that contains links to parameter_definitions resources that
     describe the parameters accepted by this resource on an HTTP POST
     method."""

    def __init__(self, **kwds):
        super(Assemblies, self).__init__(**kwds)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp.v1_1 import parameter_definitions as pd
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
class TestParameterDefinitions(base.BaseTestCase):
    def setUp(self):
        super(TestParameterDefinitions, self).setUp()
        objects.load()

    # These tests aren't strictly "unit tests" since we don't stub-out the
    # handler for parameter_definition resources. However, since that handler
    # simply looks up a static object in a static dictionary, it isn't that big
    # of a deal.

    def test_deploy_params_get(self, resp_mock, request_mock):
        cont = pd.ParameterDefinitionsController()
        resp = cont.get_one('deploy_params')
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual('parameter_definitions', resp['result'].type)
        self.assertEqual('Solum_CAMP_deploy_parameters', resp['result'].name)

    def test_ndt_params_get(self, resp_mock, request_mock):
        cont = pd.ParameterDefinitionsController()
        resp = cont.get_one('ndt_params')
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual('parameter_definitions', resp['result'].type)
        self.assertEqual('Solum_CAMP_NDT_parameters', resp['result'].name)

    def test_parameter_def_get_not_found(self, resp_mock, request_mock):
        cont = pd.ParameterDefinitionsController()
        resp = cont.get_one('does_not_exist')
        self.assertIsNotNone(resp)
        self.assertEqual(404, resp_mock.status)



# Copyright 2015 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Worker handler."""

from oslo_log import log as logging

from solum.common import exception
from solum.conductor import api as conductor_api
from solum.deployer import api as deployer_api
from solum import objects
from solum.objects import assembly
from solum.objects import image
from solum.worker.app_handlers import default as docker_handler


LOG = logging.getLogger(__name__)

ASSEMBLY_STATES = assembly.States
IMAGE_STATES = image.States


def job_update_notification(ctxt, build_id, status=None, description=None,
                            created_image_id=None, docker_image_name=None,
                            assembly_id=None):
    """send a status update to the conductor."""
    conductor_api.API(context=ctxt).build_job_update(build_id, status,
                                                     description,
                                                     created_image_id,
                                                     docker_image_name,
                                                     assembly_id)


def update_assembly_status(ctxt, assembly_id, status):
    if assembly_id is None:
        return
    data = {'status': status}
    conductor_api.API(context=ctxt).update_assembly(assembly_id, data)


def update_lp_status(ctxt, image_id, status, external_ref=None,
                     docker_image_name=None):
    if image_id is None:
        return
    LOG.debug('Updating languagepack %s status to %s and external_ref to %s'
              % (image_id, status, external_ref))
    conductor_api.API(context=ctxt).update_image(image_id, status,
                                                 external_ref,
                                                 docker_image_name)


def validate_lp(ctxt, lp_id, assembly_id):
    try:
        image = objects.registry.Image.get_lp_by_name_or_uuid(
            ctxt, lp_id, include_operators_lp=True)
    except exception.ObjectNotFound:
        LOG.error('LP not found with id %s, assembly id: %s' %
                  (lp_id, assembly_id))
        return

    if (not image or not image.project_id or not image.status or
        not image.external_ref or not image.docker_image_name or
            image.status.lower() != 'ready'):
        LOG.warning("Error building due to language pack %s not ready."
                    " assembly ID: %s" % (lp_id, assembly_id))
        return

    return image


class Handler(object):
    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    def launch_workflow(self, ctxt, build_id, git_info, ports, name,
                        base_image_id, source_format, image_format,
                        assembly_id, workflow, test_cmd, run_cmd):
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt, assembly_id)
        except exception.ObjectNotFound:
            return

        enable_unittest = enable_build = enable_deploy = False
        for step in workflow:
            if step == 'unittest':
                enable_unittest = True
            elif step == 'build':
                enable_build = True
            elif step == 'deploy':
                enable_deploy = True

        du_image_loc = None
        du_image_name = None
        with docker_handler.DockerHandler(ctxt, assem, 'custom',
                                          'swift') as lp_handler:
            if enable_unittest:
                if self._do_unittest(ctxt, lp_handler, build_id, git_info,
                                     name, base_image_id, source_format,
                                     image_format, assembly_id, test_cmd) != 0:
                    return
            if enable_build:
                du_image_loc, du_image_name = self._do_build(
                    ctxt, lp_handler, build_id, git_info, name, base_image_id,
                    source_format, image_format, assembly_id, run_cmd)

        if enable_deploy and du_image_loc and du_image_name:
            self._do_deploy(ctxt, assembly_id, ports, du_image_loc,
                            du_image_name)

    def build(self, ctxt, build_id, git_info, name, base_image_id,
              source_format, image_format, assembly_id, run_cmd):
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt, assembly_id)
        except exception.ObjectNotFound:
            return

        with docker_handler.DockerHandler(ctxt, assem, 'custom',
                                          'swift') as lp_handler:
            self._do_build(ctxt, lp_handler, build_id, git_info, name,
                           base_image_id, source_format, image_format,
                           assembly_id, run_cmd)

    def unittest(self, ctxt, build_id, git_info, name, base_image_id,
                 source_format, image_format, assembly_id, test_cmd):
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt, assembly_id)
        except exception.ObjectNotFound:
            return

        with docker_handler.DockerHandler(ctxt, assem, 'custom',
                                          'swift') as lp_handler:
            self._do_unittest(ctxt, lp_handler, build_id, git_info, name,
                              base_image_id, source_format, image_format,
                              assembly_id, test_cmd)

    def _do_deploy(self, ctxt, assembly_id, ports, du_image_loc,
                   du_image_name):
        deployer_api.API(context=ctxt).deploy(assembly_id=assembly_id,
                                              image_loc=du_image_loc,
                                              image_name=du_image_name,
                                              ports=ports)

    def _do_build(self, ctxt, lp_handler, build_id, git_info, name,
                  base_image_id, source_format, image_format, assembly_id,
                  run_cmd):
        lp = validate_lp(ctxt, base_image_id, assembly_id)
        if not lp:
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description='language pack not ready',
                                    assembly_id=assembly_id)
            return

        # Check if the assembly is deleted or being deleted
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt, assembly_id)
            if assem.status == ASSEMBLY_STATES.DELETING:
                LOG.debug('Assembly %s is being deleted..skipping next stages'
                          % assembly_id)
                return
        except exception.ObjectNotFound:
            LOG.debug('Assembly %s was deleted, skipping building.' %
                      assembly_id)
            return

        update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.BUILDING)
        image_info = lp_handler.build_app(name, git_info, lp.external_ref,
                                          lp.docker_image_name, run_cmd)
        if not image_info:
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description='image not created',
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)
            return
        else:
            job_update_notification(ctxt, build_id, IMAGE_STATES.READY,
                                    description='built successfully',
                                    created_image_id=image_info[0],
                                    docker_image_name=image_info[1],
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.BUILT)
            return (image_info[0], image_info[1])

    def _do_unittest(self, ctxt, lp_handler, build_id, git_info, name,
                     base_image_id, source_format, image_format, assembly_id,
                     test_cmd):
        if test_cmd is None:
            LOG.debug("Unit test command is None; skipping unittests.")
            return 0

        lp = validate_lp(ctxt, base_image_id, assembly_id)
        if not lp:
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)
            return -1

        # Check if the assembly is deleted or being deleted
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt, assembly_id)
            if assem.status == ASSEMBLY_STATES.DELETING:
                LOG.debug('Assembly %s is being deleted..skipping next stages'
                          % assembly_id)
                return -1
        except exception.ObjectNotFound:
            LOG.debug('Assembly %s was deleted, skipping unittesting.' %
                      assembly_id)
            return -1

        update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.UNIT_TESTING)
        result = lp_handler.unittest_app(git_info, lp.external_ref,
                                         lp.docker_image_name, test_cmd)
        if result == 0:
            status = ASSEMBLY_STATES.UNIT_TESTING_PASSED
        elif result > 0:
            status = ASSEMBLY_STATES.UNIT_TESTING_FAILED
        else:
            status = ASSEMBLY_STATES.ERROR

        update_assembly_status(ctxt, assembly_id, status)
        return result

    def build_lp(self, ctxt, image_id, git_info, name, source_format,
                 image_format, artifact_type):
        try:
            lp = objects.registry.Image.get_by_id(ctxt, image_id)
        except exception.ObjectNotFound:
            LOG.error('Image object not found with id %s' % image_id)
            return

        update_lp_status(ctxt, image_id, IMAGE_STATES.BUILDING)
        lp.type = 'languagepack'

        image_info = None
        with docker_handler.DockerHandler(ctxt, lp, 'custom', 'swift') as lph:
            image_info = lph.build_lp(name, git_info)
        if image_info:
            status = IMAGE_STATES.READY
            update_lp_status(ctxt, image_id, status, image_info[0],
                             image_info[1])
        else:
            status = IMAGE_STATES.ERROR
            update_lp_status(ctxt, image_id, status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from solum.api.controllers.camp.v1_1.datamodel import format
from solum.api.handlers import handler


GLOBAL_FORMATS = {
    'json_format':
        format.Format(uri='json_format',
                      name='JSON',
                      type='format',
                      description="JavaScript Object Notation",
                      mime_type='application/json',
                      version='RFC4627',
                      documentation='http://www.ietf.org/rfc/rfc4627.txt')
}


class FormatHandler(handler.Handler):

    def get(self, path):
        if path in GLOBAL_FORMATS:
            return GLOBAL_FORMATS[path]



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import json
import os.path
from unittest import mock

from oslo_config import cfg
from oslo_utils import uuidutils

from solum.common import exception
from solum.i18n import _
from solum.privileged import rootwrap as priv_rootwrap
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils
from solum.worker.handlers import shell as shell_handler


def mock_environment():
    return {
        'PATH': '/bin',
        'SOLUM_TASK_DIR': '/dev/null',
        'BUILD_ID': 'abcd',
        'PROJECT_ID': 1,
    }


def mock_git_info():
    return {
        'source_url': 'git://example.com/foo',
        'repo_token': '8765',
        'status_url': 'https://api.github.com/repos/u/r/statuses/SHA',
        'commit_sha': '1xxyz2'
    }


def mock_request_hdr(token):
    return {'Authorization': 'token ' + token,
            'Content-Type': 'application/json'}


def mock_req_pending_body(log_url):
    data = {'state': 'pending',
            'description': 'Solum says: Testing in progress',
            'target_url': log_url}
    return json.dumps(data)


def mock_req_success_body(log_url):
    data = {'state': 'success',
            'description': 'Solum says: Tests passed',
            'target_url': log_url}
    return json.dumps(data)


def mock_req_failure_body(log_url):
    data = {'state': 'failure',
            'description': 'Solum says: Tests failed',
            'target_url': log_url}
    return json.dumps(data)


def mock_http_response():
    return {'status': '401'}, ''


class HandlerTest(base.BaseTestCase):
    scenarios = [
        ('auto_lp_id',
         dict(base_image_id='auto',
              expected_img_id='auto',
              img_name='')),
        ('lp_id',
         dict(base_image_id='1-2-3-4',
              expected_img_id='TempUrl',
              img_name='tenant-name-ts-commit'))]

    def setUp(self):
        super(HandlerTest, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_du_details_glance(self, mock_client):
        handler = shell_handler.Handler()
        du_id = 'dummy_du_id'
        cfg.CONF.set_override('image_storage', 'glance',
                              group='worker')
        fake_du = fakes.FakeImage()
        fake_du.id = 2
        fake_du.name = 'name'

        mock_glance = mock_client.return_value.glance
        mock_get = mock_glance.return_value.images.get
        mock_get.return_value = fake_du

        du_loc, du_name = handler.get_du_details(self.ctx, du_id)

        self.assertTrue(mock_get.called)
        self.assertTrue(du_loc, 2)
        self.assertTrue(du_name, 'name')

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_du_details_GLANCE(self, mock_client):
        handler = shell_handler.Handler()
        du_id = 'dummy_du_id'
        cfg.CONF.set_override('image_storage', 'GLANCE',
                              group='worker')
        fake_du = fakes.FakeImage()
        fake_du.id = 2
        fake_du.name = 'name'

        mock_glance = mock_client.return_value.glance
        mock_get = mock_glance.return_value.images.get
        mock_get.return_value = fake_du

        du_loc, du_name = handler.get_du_details(self.ctx, du_id)

        self.assertTrue(mock_get.called)
        self.assertTrue(du_loc, 2)
        self.assertTrue(du_name, 'name')

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_du_details_swift(self, mock_client):
        handler = shell_handler.Handler()
        du_id = 'dummy_du_id'
        cfg.CONF.set_override('image_storage', 'swift',
                              group='worker')
        try:
            handler.get_du_details(self.ctx, du_id)
            self.assertTrue(False)
        except exception.NotImplemented:
            self.assertTrue(True)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.deployer.api.API.deploy')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.TLS')
    @mock.patch('solum.worker.handlers.shell.get_lp_access_method')
    def test_build(self, mock_lp_access_method, mock_trace, mock_popen,
                   mock_deploy, mock_b_update, mock_uas,
                   mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=%s\ndocker_image_name=%s' %
            (fake_glance_id, fake_image_name), None]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()

        handler.build(self.ctx, build_id=5, git_info=git_info,
                      name='new_app', base_image_id=self.base_image_id,
                      source_format='heroku', image_format='docker',
                      assembly_id=44, run_cmd=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env,
                                           stdout=-1)
        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'READY',
                              description='built successfully',
                              created_image_id=fake_glance_id,
                              docker_image_name=fake_image_name,
                              assembly_id=44)]

        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'BUILT')]
        self.assertEqual(expected, mock_uas.call_args_list)

        assert not mock_deploy.called

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.deployer.api.API.deploy')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.TLS')
    @mock.patch('solum.worker.handlers.shell.get_lp_access_method')
    def test_build_swft(self, mock_lp_access_method, mock_trace, mock_popen,
                        mock_deploy,
                        mock_b_update, mock_uas,
                        mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_uuid.return_value = fake_image
        mock_registry.Image.get_lp_by_name_or_uuid = fake_image

        cfg.CONF.api.operator_project_id = "abc"

        cfg.CONF.set_override('image_storage', 'swift',
                              group='worker')

        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=%s\ndocker_image_name=%s' %
            (fake_glance_id, fake_image_name)]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()

        handler.build(self.ctx, build_id=5, git_info=git_info, name='new_app',
                      base_image_id=fake_image.base_image_id,
                      source_format='heroku', image_format='docker',
                      assembly_id=44, run_cmd=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        expected_loc = fake_image.external_ref
        expected_tag = fake_image.docker_image_name
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            expected_loc, expected_tag],
                                           env=test_env,
                                           stdout=-1)
        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'READY',
                              description='built successfully',
                              created_image_id=fake_glance_id,
                              docker_image_name=fake_image_name,
                              assembly_id=44)]

        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'BUILT')]
        self.assertEqual(expected, mock_uas.call_args_list)

        assert not mock_deploy.called

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.worker.handlers.shell.Handler._do_deploy')
    @mock.patch('subprocess.Popen')
    @mock.patch('ast.literal_eval')
    @mock.patch('solum.TLS')
    @mock.patch('solum.worker.handlers.shell.get_lp_access_method')
    def test_build_with_private_github_repo(
            self, mock_trace, mock_lp_access, mock_ast, mock_popen,
            mock_deploy, mock_uas, mock_b_update,
            mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        handler._update_assembly_status = mock.MagicMock()
        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=%s\ndocker_image_name=%s' %
            (fake_glance_id, fake_image_name), None]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        mock_ast.return_value = [{'source_url': 'git://example.com/foo',
                                  'private_key': 'some-private-key'}]
        git_info = mock_git_info()
        handler.launch_workflow(
            self.ctx, build_id=5, git_info=git_info,
            workflow=['unittest', 'build', 'deploy'], ports=[80],
            name='new_app', base_image_id=self.base_image_id,
            source_format='heroku', image_format='docker', assembly_id=44,
            test_cmd=None, run_cmd=None, du_id=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env, stdout=-1)
        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'READY',
                              description='built successfully',
                              created_image_id=fake_glance_id,
                              docker_image_name=fake_image_name,
                              assembly_id=44)]

        self.assertEqual(expected, mock_uas.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'BUILT')]
        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, [80], fake_glance_id,
                              fake_image_name
                              )]
        self.assertEqual(expected, mock_deploy.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.worker.handlers.shell.Handler._do_deploy')
    @mock.patch('subprocess.Popen')
    @mock.patch('shelve.open')
    @mock.patch('ast.literal_eval')
    @mock.patch('solum.TLS')
    def test_build_with_private_github_repo_with_shelve(
            self, mock_trace, mock_ast, mock_shelve, mock_popen,
            mock_deploy, mock_uas, mock_b_update, mock_registry,
            mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        handler._update_assembly_status = mock.MagicMock()
        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=%s\ndocker_image_name=%s' %
            (fake_glance_id, fake_image_name), None]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        cfg.CONF.api.system_param_store = 'local_file'
        cfg.CONF.api.system_param_file = 'some_file_path'
        cfg.CONF.api.operator_project_id = "abc"
        mock_shelve.return_value = mock.MagicMock()
        base64.b64decode = mock.MagicMock()
        mock_ast.return_value = [{'source_url': 'git://example.com/foo',
                                  'private_key': 'some-private-key'}]

        git_info = mock_git_info()
        handler.launch_workflow(
            self.ctx, build_id=5, git_info=git_info,
            workflow=['unitetst', 'build', 'deploy'], ports=[80],
            name='new_app', base_image_id=self.base_image_id,
            source_format='heroku', image_format='docker', assembly_id=44,
            test_cmd=None, run_cmd=None, du_id=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        # TODO(datsun180b): Determine if this commented line should be removed
        # since I can't seem to find anywhere in shell.py that writes to
        # shelve.
        # self.assertTrue(mock_shelve.call().__setitem__.called)
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env, stdout=-1)
        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'READY',
                              description='built successfully',
                              created_image_id=fake_glance_id,
                              docker_image_name=fake_image_name,
                              assembly_id=44)]

        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'BUILT')]
        self.assertEqual(expected, mock_uas.call_args_list)

        expected = [mock.call(self.ctx, 44, [80], fake_glance_id,
                              fake_image_name
                              )]
        self.assertEqual(expected, mock_deploy.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.TLS')
    def test_build_fail(self, mock_trace, mock_popen, mock_uas,
                        mock_b_update,
                        mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=\n', None]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"

        handler.build(self.ctx, build_id=5, git_info=git_info, name='new_app',
                      base_image_id='auto', source_format='heroku',
                      image_format='docker', assembly_id=44, run_cmd=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            'auto',
                                            ''],
                                           env=test_env, stdout=-1)

        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'ERROR',
                              description='image not created',
                              assembly_id=44)]

        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'ERROR')]
        self.assertEqual(expected, mock_uas.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.TLS')
    def test_build_error(self, mock_trace, mock_popen, mock_uas,
                         mock_b_update,
                         mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_popen.call.return_value = ValueError
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"
        handler.build(self.ctx, build_id=5, git_info=git_info, name='new_app',
                      base_image_id=self.base_image_id, source_format='heroku',
                      image_format='docker', assembly_id=44, run_cmd=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            'new_app', self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env, stdout=-1)

        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'ERROR',
                              description='image not created',
                              assembly_id=44)]

        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'ERROR')]
        self.assertEqual(expected, mock_uas.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.TLS')
    def test_unittest(self, mock_trace, mock_a_update, mock_popen,
                      mock_registry,
                      mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        mock_popen.return_value.wait.return_value = 0
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"
        handler.unittest(self.ctx, build_id=5, name='new_app',
                         base_image_id=self.base_image_id,
                         source_format='chef', image_format='docker',
                         assembly_id=fake_assembly.id,
                         git_info=git_info, test_cmd='tox')

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir,
                              'contrib/lp-chef/docker/unittest-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env, stdout=-1)
        expected = [mock.call(self.ctx, 8, 'UNIT_TESTING'),
                    mock.call(self.ctx, 8, 'UNIT_TESTING_PASSED')]

        self.assertEqual(expected, mock_a_update.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.TLS')
    def test_unittest_failure(self, mock_trace, mock_a_update, mock_popen,
                              mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        mock_popen.return_value.wait.return_value = 1
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"

        handler.unittest(self.ctx, build_id=5, name='new_app',
                         assembly_id=fake_assembly.id,
                         base_image_id=self.base_image_id,
                         source_format='chef',
                         image_format='docker',
                         git_info=git_info, test_cmd='tox')

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir,
                              'contrib/lp-chef/docker/unittest-app')
        mock_popen.assert_called_once_with([script, 'git://example.com/foo',
                                            git_info['commit_sha'],
                                            self.ctx.project_id,
                                            self.expected_img_id,
                                            self.img_name],
                                           env=test_env, stdout=-1)
        expected = [mock.call(self.ctx, 8, 'UNIT_TESTING'),
                    mock.call(self.ctx, 8, 'UNIT_TESTING_FAILED')]

        self.assertEqual(expected, mock_a_update.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.worker.handlers.shell.job_update_notification')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.worker.handlers.shell.Handler._do_deploy')
    @mock.patch('solum.TLS')
    def test_unittest_build_deploy(self, mock_trace, mock_deploy,
                                   mock_a_update,
                                   mock_b_update, mock_popen, mock_registry,
                                   mock_get_env):
        handler = shell_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_popen.return_value.wait.return_value = 0
        mock_popen.return_value.communicate.return_value = [
            'foo\ncreated_image_id=%s\ndocker_image_name=%s' %
            (fake_glance_id, fake_image_name), None]
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"
        handler.launch_workflow(
            self.ctx, build_id=5, git_info=git_info,
            workflow=['unittest', 'build', 'deploy'], ports=[80],
            name='new_app', base_image_id=self.base_image_id,
            source_format='heroku', image_format='docker', assembly_id=44,
            test_cmd='faketests', run_cmd=None, du_id=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        util_dir = os.path.join(proj_dir, 'contrib', 'lp-cedarish', 'docker')
        u_script = os.path.join(util_dir, 'unittest-app')
        b_script = os.path.join(util_dir, 'build-app')

        expected = [
            mock.call([u_script, 'git://example.com/foo',
                       git_info['commit_sha'],
                       self.ctx.project_id, self.expected_img_id,
                       self.img_name], env=test_env,
                      stdout=-1),
            mock.call([b_script, 'git://example.com/foo',
                       git_info['commit_sha'], 'new_app',
                       self.ctx.project_id, self.expected_img_id,
                       self.img_name], env=test_env,
                      stdout=-1)]
        self.assertEqual(expected, mock_popen.call_args_list)

        expected = [mock.call(self.ctx, 5, 'BUILDING',
                              description='Starting the image build',
                              assembly_id=44),
                    mock.call(self.ctx, 5, 'READY',
                              description='built successfully',
                              created_image_id=fake_glance_id,
                              docker_image_name=fake_image_name,
                              assembly_id=44)]
        self.assertEqual(expected, mock_b_update.call_args_list)

        expected = [mock.call(self.ctx, 44, 'UNIT_TESTING'),
                    mock.call(self.ctx, 44, 'UNIT_TESTING_PASSED'),
                    mock.call(self.ctx, 44, 'BUILDING'),
                    mock.call(self.ctx, 44, 'BUILT')]
        self.assertEqual(expected, mock_a_update.call_args_list)

        expected = [mock.call(self.ctx, 44, [80], fake_glance_id,
                              fake_image_name
                              )]
        self.assertEqual(expected, mock_deploy.call_args_list)

    @mock.patch('solum.worker.handlers.shell.Handler._do_build')
    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('subprocess.Popen')
    @mock.patch('solum.worker.handlers.shell.update_assembly_status')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.TLS')
    def test_unittest_no_build(self, mock_trace,
                               mock_registry, mock_a_update, mock_popen,
                               mock_get_env, mock_do_build):
        handler = shell_handler.Handler()
        mock_assembly = mock.MagicMock()
        mock_registry.Assembly.get_by_id.return_value = mock_assembly
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_popen.return_value.wait.return_value = 1
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"
        handler.launch_workflow(
            self.ctx, build_id=5, git_info=git_info, name='new_app',
            base_image_id=self.base_image_id, source_format='chef',
            image_format='docker', assembly_id=44, ports=[80],
            test_cmd='faketests', run_cmd=None, workflow=['unittest', 'build'],
            du_id=None)

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        util_dir = os.path.join(proj_dir, 'contrib', 'lp-chef', 'docker')
        u_script = os.path.join(util_dir, 'unittest-app')

        expected = [
            mock.call([u_script, 'git://example.com/foo',
                       git_info['commit_sha'],
                       self.ctx.project_id, self.expected_img_id,
                       self.img_name], env=test_env,
                      stdout=-1)]
        self.assertEqual(expected, mock_popen.call_args_list)

        expected = [mock.call(self.ctx, 44, 'UNIT_TESTING'),
                    mock.call(self.ctx, 44, 'UNIT_TESTING_FAILED')]
        self.assertEqual(expected, mock_a_update.call_args_list)

        assert not mock_do_build.called


class HandlerUtilityTest(base.BaseTestCase):
    def setUp(self):
        super(HandlerUtilityTest, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch('solum.worker.handlers.shell.LOG')
    def test_echo(self, fake_LOG):
        shell_handler.Handler().echo({}, 'foo')
        fake_LOG.debug.assert_called_once_with(_('%s') % 'foo')

    @mock.patch('solum.worker.handlers.shell.get_parameter_by_assem_id')
    @mock.patch('builtins.open')
    @mock.patch('os.makedirs')
    def test_get_parameter_files(self, mock_mkdirs, mock_open, mock_param):
        mock_param.return_value = {"user_params": {'key': 'ab"cd'}}
        fake_build_id = '1-2-3-4'
        cfg.CONF.set_override('param_file_path', '/tmp/test', group='worker')
        path = '/tmp/test/' + fake_build_id
        handler = shell_handler.Handler()

        handler._get_parameter_env(self.ctx, 'git://example.com/foo',
                                   8, fake_build_id)

        mock_mkdirs.assert_called_once_with(path, 0o700)
        expected = [mock.call(path + '/user_params', 'w'),
                    mock.call(path + '/solum_params', 'w')]
        self.assertEqual(expected, mock_open.call_args_list)

        mock_file = mock_open.return_value.__enter__.return_value
        expected_params = [mock.call('#!/bin/bash\n'),
                           mock.call('export key="ab\\"cd"\n'),
                           mock.call('#!/bin/bash\n')]
        self.assertEqual(expected_params, mock_file.write.call_args_list)


class TestNotifications(base.BaseTestCase):
    def setUp(self):
        super(TestNotifications, self).setUp()
        self.ctx = utils.dummy_context()
        self.db = self.useFixture(utils.Database())

    @mock.patch('solum.conductor.api.API.update_assembly')
    @mock.patch('solum.objects.registry')
    def test_update_assembly_status(self, mock_registry, mock_uas):
        mock_assembly = mock.MagicMock()
        mock_registry.Assembly.get_by_id.return_value = mock_assembly
        shell_handler.update_assembly_status(self.ctx, '1234',
                                             'BUILDING')
        self.assertEqual(mock_registry.Assembly.get_by_id.call_count, 0)
        self.assertEqual(mock_registry.save.call_count, 0)
        self.assertEqual(mock_uas.call_count, 1)

    @mock.patch('solum.conductor.api.API.update_assembly')
    @mock.patch('solum.objects.registry')
    def test_update_assembly_status_pass(self, mock_registry, mock_uas):
        shell_handler.update_assembly_status(self.ctx, None,
                                             'BUILDING')
        self.assertEqual(mock_registry.call_count, 0)


class TestBuildCommand(base.BaseTestCase):
    scenarios = [
        ('docker',
         dict(source_format='heroku', image_format='docker',
              base_image_id='auto', artifact_type=None,
              expect_b='lp-cedarish/docker/build-app',
              expect_u='lp-cedarish/docker/unittest-app')),
        ('dockerfile',
         dict(source_format='dockerfile', image_format='docker',
              base_image_id='auto', artifact_type=None,
              expect_b='lp-dockerfile/docker/build-app',
              expect_u='lp-dockerfile/docker/unittest-app')),
        ('chef',
         dict(source_format='chef', image_format='docker',
              base_image_id='xyz', artifact_type=None,
              expect_b='lp-chef/docker/build-app',
              expect_u='lp-chef/docker/unittest-app'))]

    def test_build_cmd(self):
        ctx = utils.dummy_context()
        handler = shell_handler.Handler()
        cmd = handler._get_build_command(ctx,
                                         'build',
                                         'http://example.com/a.git',
                                         'testa',
                                         self.base_image_id,
                                         self.source_format,
                                         self.image_format, '',
                                         self.artifact_type)
        self.assertIn(self.expect_b, cmd[0])
        self.assertEqual('http://example.com/a.git', cmd[1])
        self.assertEqual('testa', cmd[3])
        self.assertEqual(ctx.project_id, cmd[4])
        if self.base_image_id == 'auto' and self.image_format == 'qcow2':
            self.assertEqual('cedarish', cmd[5])
        else:
            self.assertEqual(self.base_image_id, cmd[5])

    def test_unittest_cmd(self):
        ctx = utils.dummy_context()
        handler = shell_handler.Handler()
        cmd = handler._get_build_command(ctx,
                                         'unittest',
                                         'http://example.com/a.git',
                                         'testa',
                                         self.base_image_id,
                                         self.source_format,
                                         self.image_format, 'asdf',
                                         self.artifact_type)
        self.assertIn(self.expect_u, cmd[0])
        self.assertEqual('http://example.com/a.git', cmd[1])
        self.assertEqual('asdf', cmd[2])
        self.assertEqual(ctx.project_id, cmd[3])


class TestLanguagePackBuildCommand(base.BaseTestCase):
    def setUp(self):
        super(TestLanguagePackBuildCommand, self).setUp()
        self.ctx = utils.dummy_context()

    def test_languagepack_build_cmd(self):
        ctx = utils.dummy_context()
        handler = shell_handler.Handler()
        cmd = handler._get_build_command(ctx,
                                         'build',
                                         'http://example.com/a.git',
                                         'testa',
                                         'auto',
                                         'heroku',
                                         'docker', '',
                                         'language_pack')
        self.assertIn('lp-cedarish/docker/build-lp', cmd[0])
        self.assertEqual('http://example.com/a.git', cmd[1])
        self.assertEqual('testa', cmd[2])
        self.assertEqual(ctx.project_id, cmd[3])

    @mock.patch('solum.worker.handlers.shell.Handler._get_environment')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.worker.handlers.shell.update_lp_status')
    @mock.patch.object(priv_rootwrap, 'execute')
    @mock.patch('solum.TLS')
    def test_build_lp(self, mock_trace, mock_execute, mock_ui,
                      mock_registry, mock_get_env):
        handler = shell_handler.Handler()
        fake_image = fakes.FakeImage()
        fake_glance_id = uuidutils.generate_uuid()
        fake_image_name = 'tenant-name-ts-commit'
        mock_registry.Image.get_lp_by_name_or_uuid.return_value = fake_image
        mock_execute.return_value = (
            'foo\nimage_external_ref=%s\ndocker_image_name=%s\n' %
            (fake_glance_id, fake_image_name), None)
        test_env = mock_environment()
        mock_get_env.return_value = test_env
        git_info = mock_git_info()
        cfg.CONF.api.operator_project_id = "abc"
        handler.build_lp(self.ctx, image_id=5, git_info=git_info,
                         name='lp_name', source_format='heroku',
                         image_format='docker', artifact_type='language_pack',
                         lp_params='')

        proj_dir = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '..', '..', '..', '..'))
        script = os.path.join(proj_dir, 'contrib/lp-cedarish/docker/build-lp')
        mock_execute.assert_called_once_with(
            script, 'git://example.com/foo',
            'lp_name', self.ctx.project_id,
            env_variables=test_env,
            run_as_root=True,
        )

        expected = [mock.call(self.ctx, 5, 'lp_name', 'BUILDING'),
                    mock.call(self.ctx, 5, 'lp_name', 'READY',
                              fake_glance_id, fake_image_name)]
        self.assertEqual(expected, mock_ui.call_args_list)



# Copyright 2014 - Numergy
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import infrastructure as inframodel
from solum.api.controllers.v1 import infrastructure
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
class TestInfrastructureController(base.BaseTestCase):

    def test_stack_get(self, resp_mock, request_mock):
        cont = infrastructure.InfrastructureController()
        res = cont.index()
        self.assertIsNotNone(res)
        self.assertEqual(res['result'].uri, "http://test_url:8080/test/v1")
        self.assertEqual(res['result'].name, "solum")
        self.assertEqual(res['result'].type, "infrastructure")
        self.assertEqual(res['result'].description,
                         "solum infrastructure endpoint")
        self.assertEqual(res['result'].stacks_uri,
                         "http://test_url:8080/test/v1/infrastructure/stacks")


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.infrastructure_handler.'
            'InfrastructureStackHandler')
class TestInfrastructureStackController(base.BaseTestCase):

    def setUp(self):
        super(TestInfrastructureStackController, self).setUp()
        objects.load()

    def test_stack_get(self, StackHandler, resp_mock, request_mock):
        hand_get = StackHandler.return_value.get
        fake_stack = fakes.FakeInfrastructureStack()
        hand_get.return_value = fake_stack
        cont = infrastructure.InfrastructureStackController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_stack.name, resp['result'].name)
        self.assertEqual(fake_stack.image_id, resp['result'].image_id)
        self.assertEqual(fake_stack.heat_stack_id,
                         resp['result'].heat_stack_id)
        self.assertEqual(fake_stack.description,
                         resp['result'].description)
        self.assertEqual(fake_stack.project_id, resp['result'].project_id)
        self.assertEqual(fake_stack.uuid, resp['result'].uuid)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_stack_get_not_found(self, StackHandler, resp_mock,
                                 request_mock):
        hand_get = StackHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(
            id='test_id', name='stack')
        cont = infrastructure.InfrastructureStackController('test_id')
        cont.get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_stack_put_none(self, StackHandler, resp_mock,
                            request_mock):
        request_mock.body = None
        request_mock.content_type = 'application/json'
        hand_put = StackHandler.return_value.put
        hand_put.return_value = fakes.FakeInfrastructureStack()
        infrastructure.InfrastructureStackController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_stack_put_not_found(self, StackHandler, resp_mock,
                                 request_mock):
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = StackHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            id='test_id', name='stack')
        infrastructure.InfrastructureStackController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_stack_put_ok(self, StackHandler, resp_mock,
                          request_mock):
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = StackHandler.return_value.update
        hand_update.return_value = fakes.FakeInfrastructureStack()
        infrastructure.InfrastructureStackController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(200, resp_mock.status)

    def test_stack_delete_not_found(self, StackHandler,
                                    resp_mock, request_mock):
        hand_delete = StackHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            id='test_id', name='stack')
        obj = infrastructure.InfrastructureStackController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_stack_delete_ok(self, StackHandler, resp_mock,
                             request_mock):
        hand_delete = StackHandler.return_value.delete
        hand_delete.return_value = None
        obj = infrastructure.InfrastructureStackController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.infrastructure_handler.'
            'InfrastructureStackHandler')
class TestStacksController(base.BaseTestCase):
    def setUp(self):
        super(TestStacksController, self).setUp()
        objects.load()

    def test_stacks_get_all(self, handler_mock, resp_mock, request_mock):
        hand_get_all = handler_mock.return_value.get_all
        fake_stack = fakes.FakeInfrastructureStack()
        hand_get_all.return_value = [fake_stack]
        obj = infrastructure.InfrastructureStacksController()
        resp = obj.get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_stack.name, resp['result'][0].name)
        self.assertEqual(fake_stack.image_id, resp['result'][0].image_id)
        self.assertEqual(fake_stack.heat_stack_id,
                         resp['result'][0].heat_stack_id)
        self.assertEqual(fake_stack.description,
                         resp['result'][0].description)
        self.assertEqual(fake_stack.project_id,
                         resp['result'][0].project_id)
        self.assertEqual(fake_stack.uuid, resp['result'][0].uuid)
        self.assertEqual(200, resp_mock.status)

    def test_stacks_post(self, handler_mock, resp_mock, request_mock):
        json_create = {'name': 'foo',
                       'description': 'test_desc_stack',
                       'user_id': 'user_id_test',
                       'project_id': 'project_id_test'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeInfrastructureStack()
        infrastructure.InfrastructureStacksController().post()
        handler_create.assert_called_with(json_create)
        self.assertEqual(201, resp_mock.status)
        handler_create.assert_called_once_with(json_create)

    def test_stacks_post_nodata(self, handler_mock,
                                resp_mock, request_mock):
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeComponent()
        ret_val = infrastructure.InfrastructureStacksController().post()
        self.assertEqual("Missing argument: \"data\"",
                         str(ret_val['faultstring']))
        self.assertEqual(400, resp_mock.status)


class TestStackAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/infrastructure/'
                                   'stack/x1',
                            'name': 'Example-stack',
                            'type': 'infrastructure_stack',
                            'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                            'user_id': '55f41cf46df74320b9486a35f5d28a11',
                            'image_id': '1234',
                            'heat_stack_id': '5678'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = inframodel.InfrastructureStack()
            self.data = {}
        else:
            s = inframodel.InfrastructureStack(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']
        self.assertEqual(self.data, s.as_dict(
            objects.registry.InfrastructureStack))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme.rest import json
from wsme import types as wtypes


TRUE_STRINGS = ('true', 'True', 'TRUE')
FALSE_STRINGS = ('false', 'False', 'FALSE')


class BooleanType(wtypes.UserType):
    """A simple boolean type."""

    basetype = wtypes.text
    name = 'boolean'

    @staticmethod
    def validate(value):
        if not isinstance(value, str):
            if isinstance(value, bytes):
                value = value.decode('utf-8')
            else:
                value = str(value)

        if value in TRUE_STRINGS:
            ret_val = True
        elif value in FALSE_STRINGS:
            ret_val = False
        else:
            acceptable = ', '.join(
                "'%s'" % s for s in sorted(TRUE_STRINGS + FALSE_STRINGS))
            msg = ("Unrecognized value '%(val)s', acceptable values are:"
                   " %(acceptable)s") % {'val': value,
                                         'acceptable': acceptable}
            raise ValueError(msg)

        return ret_val

    @staticmethod
    def frombasetype(value):
        if value is None:
            return None
        return BooleanType.validate(value)


@json.tojson.when_object(BooleanType)
def bool_tojson(datatype, value):
    if value:
        return 'true'
    else:
        return 'false'



# Copyright 2015 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities and helper functions for keystone integration."""

from oslo_config import cfg

from solum.common import clients
from solum.common import context


def create_delegation_context(assembly_obj, ctx=None):
    """Create a delegated security context."""
    if cfg.CONF.get('keystone_version') == '3':
        cntx = context.RequestContext(trust_id=assembly_obj.trust_id)
        kc = clients.OpenStackClients(cntx).keystone()
        return kc.context


def delete_delegation_token(user_ctx, trust_id):
    if cfg.CONF.get('keystone_version') == '3':
        ksc = clients.OpenStackClients(user_ctx).keystone()
        ksc.delete_trust(trust_id)


def create_delegation_token(user_ctx):
    if cfg.CONF.get('keystone_version') == '3':
        ksc = clients.OpenStackClients(user_ctx).keystone()
        trust_context = ksc.create_trust_context()
        return trust_context.trust_id



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add workflow column to assembly table

Revision ID: 2f15f41cd87f
Revises: 176c26131889
Create Date: 2015-03-04 23:10:57.979224

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

# revision identifiers, used by Alembic.
revision = '2f15f41cd87f'
down_revision = '176c26131889'


def upgrade():
    op.add_column('assembly',
                  sa.Column('workflow', models.YAMLEncodedDict(1024)))


def downgrade():
    op.drop_column('assembly', 'workflow')



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import string

import wsme
from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.i18n import _


class Base(wtypes.Base):
    """Base class for all API types."""

    uri = common_types.Uri
    "URI to the resource."

    # (devkulkarni) Added base_url to get around strict validation
    # checking of WSME 0.8.0
    # https://bugs.launchpad.net/solum/+bug/1491504
    # https://bugs.launchpad.net/solum/+bug/1491499
    base_url = common_types.Uri
    "URI of the base resource."

    uuid = wtypes.text
    "Unique Identifier of the resource"

    def get_name(self):
        return self.__name

    def set_name(self, value):
        allowed_chars = string.ascii_letters + string.digits + '-_'
        for ch in value:
            if ch not in allowed_chars:
                raise ValueError(_('Names must only contain a-z,A-Z,0-9,-,_'))
        self.__name = value

    name = wtypes.wsproperty(str, get_name, set_name, mandatory=True)
    "Name of the resource."

    type = wtypes.text
    "The resource type."

    description = wtypes.text
    "Textual description of the resource."

    tags = [wtypes.text]
    "Tags for the resource."

    project_id = wtypes.text
    "The project that this resource belongs in."

    user_id = wtypes.text
    "The user that owns this resource."

    def __init__(self, **kwds):
        self.__name = wsme.Unset
        super(Base, self).__init__(**kwds)

    @classmethod
    def from_db_model(cls, m, host_url):
        json = m.as_dict()
        json['type'] = m.__tablename__
        json['uri'] = '%s/v1/%s/%s' % (host_url, m.__resource__, m.uuid)
        del json['id']
        return cls(**(json))

    def as_dict(self, db_model):
        valid_keys = (attr for attr in db_model.__dict__.keys()
                      if attr[:2] != '__' and attr != 'as_dict')
        return self.as_dict_from_keys(valid_keys)

    def as_dict_from_keys(self, keys):
        return dict((k, getattr(self, k))
                    for k in keys
                    if hasattr(self, k) and
                    getattr(self, k) != wsme.Unset)


class MultiType(wtypes.UserType):
    """A complex type that represents one or more types.

    Used for validating that a value is an instance of one of the types.

    :param *types: Variable-length list of types.
    """

    def __init__(self, *types):
        self.types = types

    def __str__(self):
        return ' | '.join(map(str, self.types))

    def validate(self, value):
        for t in self.types:
            try:
                return wtypes.validate_value(t, value)
            except (ValueError, TypeError):
                pass
        else:
            raise ValueError(
                _("Wrong type. Expected '%(type)s', got '%(value)s'")
                % {'type': self.types, 'value': type(value)})


PortType = wtypes.IntegerType(minimum=1, maximum=65535)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

sys.path.insert(0, os.path.abspath('../..'))
# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinxcontrib.pecanwsme.rest',
    'sphinxcontrib.httpdomain',
    'wsmeext.sphinxext',
    'openstackdocstheme',
]

wsme_protocols = ['restjson', 'restxml']

suppress_warnings = ['app.add_directive']

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'solum'
copyright = u'2014, Solum Contributors'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
# html_theme = '_theme'
html_theme = 'openstackdocs'
html_static_path = ['_static']

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/solum'
openstackdocs_pdf_link = True
openstackdocs_auto_name = False
openstackdocs_bug_project = 'solum'
openstackdocs_bug_tag = ''

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# -- Options for manual page output -------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('man/solum-db-manage', 'solum-db-manage',
     u'Script which helps manage specific database operations',
     [u'Solum Developers'], 1),
]

# If true, show URL addresses after external links.
man_show_urls = True

# -- Options for LaTeX output -------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index', 'doc-solum.tex', u'Solum Documentation',
     u'OpenStack Foundation', 'manual'),
]
latex_domain_indices = False

latex_elements = {
    'makeindex': '',
    'printindex': '',
    'preamble': r'\setcounter{tocdepth}{3}',
    'maxlistdepth': '10',
}

# Disable usage of xindy https://bugzilla.redhat.com/show_bug.cgi?id=1643664
latex_use_xindy = False

# Disable smartquotes, they don't work in latex
smartquotes_excludes = {'builders': ['latex']}



# Copyright 2015 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Swift Handler"""

import errno
import httplib
import os
import sys

import six
from swiftclient import client as swiftclient
from swiftclient import exceptions as swiftexp


CHUNKSIZE = 65536
TOTAL_RETRIES = 3
Gi = 1024 * 1024 * 1000
LARGE_OBJECT_SIZE = 5 * Gi


class InvalidObjectSizeError(Exception):
    pass


class MaxRetryReached(Exception):
    pass


def _get_swift_client(args):
    client_args = {
        'auth_version': '2.0',
        'preauthtoken': args['auth_token'],
        'preauthurl': args['storage_url'],
        'os_options': {'region_name': args['region_name']},
        }

    # swiftclient connection will retry the request
    # 5 times before failing
    return swiftclient.Connection(**client_args)


def _get_file_size(file_obj):
    # Analyze file-like object and attempt to determine its size.

    if (hasattr(file_obj, 'seek') and hasattr(file_obj, 'tell') and
            (six.PY2 or six.PY3 and file_obj.seekable())):
        try:
            curr = file_obj.tell()
            file_obj.seek(0, os.SEEK_END)
            size = file_obj.tell()
            file_obj.seek(curr)
            return size
        except IOError as e:
            if e.errno == errno.ESPIPE:
                # Illegal seek. This means the file object
                # is a pipe (e.g. the user is trying
                # to pipe image data to the client,
                # echo testdata | bin/glance add blah...), or
                # that file object is empty, or that a file-like
                # object which doesn't support 'seek/tell' has
                # been supplied.
                return 0
            else:
                print("Error getting file size")
                raise
    else:
        return 0


def _get_object(container, name, connection_args, start_byte=None):
    connection = _get_swift_client(connection_args)
    headers = {}
    if start_byte is not None:
        bytes_range = 'bytes=%d-' % start_byte
        headers = {'Range': bytes_range}

    try:
        resp_headers, resp_body = connection.get_object(
            container=container, obj=name, resp_chunk_size=CHUNKSIZE,
            headers=headers)
    except swiftexp.ClientException as e:
        if e.http_status == httplib.NOT_FOUND:
            print("Swift could not find object %s." % name)
        raise

    return (resp_headers, resp_body)


def _retry_iter(resp_iter, length, container, name, connection_args):
    length = length if length else (resp_iter.len
                                    if hasattr(resp_iter, 'len') else 0)
    retries = 0
    bytes_read = 0

    while retries <= TOTAL_RETRIES:
        try:
            for chunk in resp_iter:
                yield chunk
                bytes_read += len(chunk)
        except swiftexp.ClientException as e:
            print("Swift exception %s" % e.__class__.__name__)

        if bytes_read == length:
            break
        else:
            if retries == TOTAL_RETRIES:
                raise MaxRetryReached
            else:
                retries += 1
                print("Retrying Swift download")
                # NOTE(james_li): Need a new swift connection to do
                # a range request for the same object
                (_resp_headers, resp_iter) = _get_object(container, name,
                                                         connection_args,
                                                         start_byte=bytes_read)


def do_upload(path, container, name, connection_args):
    connection = _get_swift_client(connection_args)
    with open(path, 'rb') as local_file:
        size = _get_file_size(local_file)
        if size > 0 and size < LARGE_OBJECT_SIZE:
            connection.put_container(container)
            connection.put_object(container, name, local_file,
                                  content_length=size)
        else:
            print("Cannot upload a file with the size exceeding 5GB")
            raise InvalidObjectSizeError


def do_download(path, container, name, connection_args):
    (resp_headers, resp_data) = _get_object(container, name, connection_args)
    length = int(resp_headers.get('content-length', 0))
    data_iter = _retry_iter(resp_data, length, container, name,
                            connection_args)

    with open(path, 'wb') as local_file:
        for chunk in data_iter:
            local_file.write(chunk)
        local_file.flush()


def main():
    action_to_take = sys.argv[4]
    path = str(sys.argv[7])
    container = str(sys.argv[5])
    obj_name = str(sys.argv[6])
    connection_args = {'region_name': str(sys.argv[1]),
                       'auth_token': str(sys.argv[2]),
                       'storage_url': str(sys.argv[3])}

    if action_to_take == 'download':
        try:
            do_download(path, container, obj_name, connection_args)
            print("Finished swift download.")
            sys.exit(0)
        except Exception as e:
            print("Error download object, got %s" % e.__class__.__name__)
            sys.exit(1)
    elif action_to_take == 'upload':
        try:
            do_upload(path, container, obj_name, connection_args)
            print("Finished swift upload.")
            sys.exit(0)
        except Exception as e:
            print("Error upload object, got %s" % e.__class__.__name__)
            sys.exit(1)
    else:
        sys.exit(2)


if __name__ == '__main__':
    sys.exit(main())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cryptography.fernet import Fernet

from oslo_config import cfg
from oslo_utils import encodeutils

from solum.privileged import rootwrap as priv_rootwrap


key = Fernet.generate_key()
cipher_suite = Fernet(key)


def encrypt(value):
    ciphertext = cipher_suite.encrypt(encodeutils.safe_encode(value))
    return ciphertext


def decrypt(ciphertext):
    value = cipher_suite.decrypt(ciphertext)
    return encodeutils.safe_decode(value, 'utf-8')


def get_root_helper():
    solum_rootwrap_config = cfg.CONF.worker.rootwrap_config
    return 'sudo solum-rootwrap %s' % solum_rootwrap_config


def execute(*cmd, **kwargs):
    """Convenience wrapper around oslo's execute() method."""
    if 'run_as_root' in kwargs and 'root_helper' not in kwargs:
        kwargs['root_helper'] = get_root_helper()
    return priv_rootwrap.execute(*cmd, **kwargs)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sqlalchemy.orm import exc as sqla_ex
from unittest import mock

from solum.common import exception
from solum.objects import registry
from solum.objects.sqlalchemy import assembly
from solum.tests import base
from solum.tests import utils

from oslo_utils import uuidutils


class TestAssembly(base.BaseTestCase):
    def setUp(self):
        super(TestAssembly, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()

        self.data = [{'project_id': self.ctx.project_id,
                      'uuid': 'ce43e347f0b0422825245b3e5f140a81cef6e65b',
                      'user_id': 'fred',
                      'name': 'assembly1',
                      'description': 'test assembly',
                      'tags': 'assembly tags',
                      'plan_id': 'plan_id_1',
                      'status': 'BUILDING',
                      'application_uri': 'http://192.168.78.21:5000'}]
        utils.create_models_from_data(assembly.Assembly, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Assembly)
        self.assertTrue(registry.AssemblyList)

    def test_get_all(self):
        lst = assembly.AssemblyList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        ta = assembly.Assembly().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(ta, key))

    def test_del_assem_with_comps(self):
        ta = registry.Assembly().get_by_id(self.ctx, self.data[0]['id'])
        comp = registry.Component()
        comp.uuid = uuidutils.generate_uuid()
        comp.name = 't'
        comp.assembly_id = ta.id
        comp.create(self.ctx)
        comp_id = comp.id
        ta.destroy(self.ctx)

        self.assertRaises(exception.ResourceNotFound,
                          registry.Assembly().get_by_id,
                          self.ctx, self.data[0]['id'])
        self.assertRaises(exception.ResourceNotFound,
                          registry.Component().get_by_id, self.ctx, comp_id)

    def test_update_and_save(self):
        # first update
        assembly.Assembly().update_and_save(self.ctx, self.data[0]['id'],
                                            {'status': 'DELETING'})
        updated = assembly.Assembly().get_by_id(self.ctx, self.data[0]['id'])
        self.assertEqual('DELETING', getattr(updated, 'status'))

        # second update, 'DELETING' status is not updatable
        assembly.Assembly().update_and_save(self.ctx, self.data[0]['id'],
                                            {'status': 'DEPLOYMENT_COMPLETE'})
        updated = assembly.Assembly().get_by_id(self.ctx, self.data[0]['id'])
        self.assertEqual('DELETING', getattr(updated, 'status'))

    @mock.patch('solum.objects.sqlalchemy.models.SolumBase.get_session')
    @mock.patch('solum.objects.sqlalchemy.models.LOG')
    def test_update_and_save_raise_exp(self, mock_log, mock_sess):
        mock_sess.return_value.merge.side_effect = sqla_ex.StaleDataError
        self.assertRaises(sqla_ex.StaleDataError,
                          assembly.Assembly().update_and_save,
                          self.ctx, self.data[0]['id'], {'status': 'DELETING'})
        expected = [mock.call("Failed DB call update_and_save. "
                              "Retrying 2 more times."),
                    mock.call("Failed DB call update_and_save. "
                              "Retrying 1 more times."),
                    mock.call("Failed DB call update_and_save. "
                              "Retrying 0 more times.")]
        self.assertEqual(expected, mock_log.warning.call_args_list)
